@@ -18,8 +18,11 @@
         parallel_states: return a list of strings naming all parallel states
         statenames: return a list of properly-formatted state names
         rec_findstates: recursively find parallel/composite statenames
+        update_full_statelist(process): set field full_statelist in process AST
+        code_generation_preprocessing: to be called before generating code
+        generate_asn1_datamodel: generate the _datamodel.asn file for a process
 
-    Copyright (c) 2012-2020 European Space Agency
+    Copyright (c) 2012-2022 European Space Agency
 
     Designed and implemented by Maxime Perrotin
 
@@ -36,20 +39,22 @@ from functools import singledispatch
 from . import ogAST
 
 LOG = logging.getLogger(__name__)
-#DEFAULT_SEPARATOR=u'\00dc'
 DEFAULT_SEPARATOR='_0_'
+ASN1SCC = 'asn1Scc'
 
 __all__ = ['flatten', 'rename_everything', 'inner_labels_to_floating',
            'map_input_state', 'sorted_fields', 'state_aggregations',
-           'parallel_states', 'statenames', 'rec_findstates']
+           'parallel_states', 'statenames', 'rec_findstates',
+           'generate_asn1_datamodel',
+           'get_full_statelist', 'code_generation_preprocessing']
 
 
 def statenames(context, sep=DEFAULT_SEPARATOR):
     ''' Return the list of states (just the names) of a given context
     Format the output by replacing unicode separator symbol with a dot '''
     # note: if model has been flattened, all contexts are already merged
-    return (s.replace(sep, u'.') for s in context.mapping.keys()
-            if not s.endswith(u'START'))
+    return (s.replace(sep, '.') for s in context.mapping.keys()
+            if not s.endswith('START'))
 
 
 def rec_findstates(context, prefix=''):
@@ -58,10 +63,9 @@ def rec_findstates(context, prefix=''):
         if not isinstance(each, ogAST.StateAggregation):
             # Aggregations are just containers, not states
             for name in statenames(each):
-                yield u'{}{}.{}'.format(prefix, each.statename, name)
+                yield f'{prefix}{each.statename}.{name}'
         for substate in rec_findstates(each,
-                                       prefix=u'{}{}.'.format(prefix,
-                                                              each.statename)):
+                                       prefix=f'{prefix}{each.statename}.'):
             yield substate
 
 
@@ -139,6 +143,19 @@ def map_input_state(process):
     return mapping
 
 
+def update_full_statelist(process, SEPARATOR=DEFAULT_SEPARATOR) -> None:
+    ''' Compute the value of the AST field "process.full_statelist"
+        Inputs:
+          * process is of type ogAST.Process
+    '''
+
+    process.full_statelist = set(chain(process.aggregates.keys(),
+                               (name for name in process.mapping.keys()
+                                    if not name.endswith('START'))))
+    if process.aggregates:
+        # Parallel states in a state aggregation may terminate
+        process.full_statelist.add(f'state{SEPARATOR}end')
+
 def inner_labels_to_floating(process):
     '''
         Transform inner labels of both floating labels and transitions
@@ -170,23 +187,22 @@ def flatten(process, sep=u'_'):
         if term.inputString.lower() in (st.statename.lower()
                                 for st in context.composite_states):
             if not term.via:
-                term.next_id = term.inputString.lower() + sep + u'START'
+                term.next_id = term.inputString.lower() + sep + 'START'
             else:
-                term.next_id = u'{term}{sep}{entry}_START'.format(
-                        term=term.inputString, entry=term.entrypoint, sep=sep)
-        elif term.inputString.strip() == '-':
+                term.next_id = f'{term.inputString}{sep}{term.entrypoint}_START'
+        elif term.inputString.strip() in ('-', '-*'):
             for each in term.possible_states:
                 term.candidate_id[-1].append(each)
                 for comp in context.composite_states:
                     if each.lower() == comp.statename.lower():
                         if isinstance(comp, ogAST.StateAggregation):
                             term.next_is_aggregation = True
-                            term.candidate_id[each + sep + u'START'] = [each]
+                            term.candidate_id[each + sep + 'START'] = [each]
                         else:
-                            term.candidate_id[each + sep + u'START'] = \
+                            term.candidate_id[each + sep + 'START'] = \
                                        [st for st in process.mapping.keys()
                                         if st.startswith(each)
-                                        and not st.endswith(u'START')]
+                                        and not st.endswith('START')]
                         continue
 
     def update_composite_state(state, process):
@@ -250,13 +266,13 @@ def flatten(process, sep=u'_'):
                          if isinstance(trans, int)):
                 call_entry = ogAST.ProcedureCall()
                 call_entry.inputString = 'entry'
-                entryproc = u'{pre}entry'.format(pre=prefix)
+                entryproc = f'{prefix}entry'
                 call_entry.output = [{'outputName': entryproc,
                                      'params': [], 'tmpVars': []}]
                 process.transitions[each].actions.insert(0, call_entry)
 
         # If composite state has exit procedure, add an call to this
-        # procedure if the transition ends up existing the state with
+        # procedure if the transition ends up exiting the state with
         # a return statement. There are other calls to the exit procedure
         # that the code generation backend must add when the state is exited
         # from a transition trigger in the super state. See AdaGenerator.py
@@ -280,7 +296,7 @@ def flatten(process, sep=u'_'):
             for trans in trans_with_return:
                 call_exit = ogAST.ProcedureCall()
                 call_exit.inputString = 'exit'
-                exitproc = u'{pre}exit'.format(pre=prefix)
+                exitproc = f'{prefix}exit'
                 call_exit.output = [{'outputName': exitproc,
                                      'params': [], 'tmpVars': []}]
                 trans.actions.append(call_exit)
@@ -297,7 +313,7 @@ def flatten(process, sep=u'_'):
             if each.label:
                 each.label.inputString = prefix + each.label.inputString
             if each.kind == 'next_state':
-                if each.inputString.strip() != '-':
+                if each.inputString.strip() not in ('-', '-*'):
                     each.inputString = prefix + each.inputString
                 # Set next transition id
                 update_terminator(context=state, term=each, process=process)
@@ -352,7 +368,9 @@ def flatten(process, sep=u'_'):
         ''' Associate state to terminators, needed to process properly
             a history nextstates (dash nextstate) in code generators '''
         for each in context.content.states:
-            for inp in each.inputs:
+            for inp in chain(each.inputs,
+                             each.continuous_signals,
+                             each.connects):
                 for term in inp.terminators:
                     term.possible_states.extend(prefix + name.lower()
                                                 for name in each.statelist)
@@ -361,7 +379,9 @@ def flatten(process, sep=u'_'):
         ''' Associate state to transitions, needed to process properly
             the call to the exit procedure of a nested state '''
         for each in context.content.states:
-            for inp in each.inputs:
+            for inp in chain(each.inputs,
+                             each.continuous_signals,
+                             each.connects):
                 inp.transition.possible_states.extend(prefix + name.lower()
                                                 for name in each.statelist)
 
@@ -382,7 +402,7 @@ def flatten(process, sep=u'_'):
 @singledispatch
 def rename_everything(ast, from_name, to_name):
     '''
-        Rename in all symbols all occurences of name_ref into new_name.
+        Rename in all symbols all occurrences of name_ref into new_name.
         This is used to avoid name clashes in nested diagrams when they get
         flattened. For example rename all accesses to a variable declared
         in the scope of a composite state, so that they do not overwrite
@@ -399,7 +419,7 @@ def rename_everything(ast, from_name, to_name):
 
 @rename_everything.register(ogAST.Automaton)
 def _rename_automaton(ast, from_name, to_name):
-    ''' Renaming at Automaton top level (content of digragrams) '''
+    ''' Renaming at Automaton top level (content of diagrams) '''
     if ast.start:
         rename_everything(ast.start.transition, from_name, to_name)
     for each in ast.named_start:
@@ -465,10 +485,12 @@ def _rename_decision(ast, from_name, to_name):
 @rename_everything.register(ogAST.Answer)
 def _rename_answer(ast, from_name, to_name):
     ''' Rename elements in an answer branch '''
-    if ast.kind in ('constant', 'open_range'):
-        rename_everything(ast.constant, from_name, to_name)
-    elif ast.kind == 'closed_range':
-        pass  # TODO when supported
+    for each in ast.answers:
+        if each['kind'] in ('constant', 'open_range'):
+            _, constant = each['content'] # get the constant
+            rename_everything(constant, from_name, to_name)
+        elif each['kind'] == 'closed_range':
+            pass  # TODO when supported
     rename_everything(ast.transition, from_name, to_name)
 
 
@@ -588,7 +610,7 @@ def _rename_primcall(ast, from_name, to_name):
 
 
 @rename_everything.register(ogAST.PrimConditional)
-def _rename_ifhthenelse(ast, from_name, to_name):
+def _rename_ifthenelse(ast, from_name, to_name):
     ''' Rename expressions in Conditional expression construct '''
     for expr in ('if', 'then', 'else'):
         rename_everything(ast.value[expr], from_name, to_name)
@@ -647,3 +669,188 @@ def sorted_fields(atype):
     tmp = ([k, val.Line, val.CharPositionInLine]
              for k, val in atype.Children.items())
     return (x[0] for x in sorted(tmp, key=operator.itemgetter(1,2)))
+
+
+def find_basic_type(TYPES, a_type):
+    ''' Return the ASN.1 basic type of a_type.
+    TYPES is process.dataview
+    '''
+    basic_type = a_type
+    while basic_type.kind == 'ReferenceType':
+        # Find type with proper case in the data view
+        for typename in TYPES.keys():
+            if typename.lower() == basic_type.ReferencedTypeName.lower():
+                basic_type = TYPES[typename].type
+                break
+    return basic_type
+
+
+def generate_asn1_datamodel(process: ogAST.Process, SEPARATOR: str=DEFAULT_SEPARATOR) -> None:
+    ''' Generate an ASN.1 model containing:
+          - the state definition
+          - a type describing the SDL context
+          - all user-defined SDL type (NEWTYPEs...)
+        This function is independent from specific backends.
+    '''
+
+    if not process.name:
+        process.name = process.processName
+
+    process_asn1 = process.name.capitalize().replace('_', '-')
+    asn1_template = [
+            f'{process_asn1}-Datamodel DEFINITIONS ::=',
+            'BEGIN',
+            '--  This file was generated automatically by OpenGEODE']
+
+    # The ASN.1 module must import the types from other asn1 modules
+    types_with_proper_case, imports = [], []
+    imports = []
+    for moduleName, sorts in process.DV.exportedTypes.items():
+        actual_sorts = []
+        for each in sorts:
+            process.mapping_sort_module[each] = moduleName
+            if process.DV.types[each].AddedType == "False":
+                actual_sorts.append(each)
+                types_with_proper_case.append(each)
+        sortsAsList = ", ".join(actual_sorts)
+        if sortsAsList:
+            imports.append(f'{sortsAsList} FROM {moduleName}')
+    if types_with_proper_case:
+        asn1_template.append(
+                "IMPORTS\n   " + "\n   ".join(imports) + ";")
+
+    # Format the state list with ASN.1-compatible syntax
+    statelist_asn1 = (state.lower().replace('_', '-')
+                      for state in process.full_statelist)
+    states_asn1 = ", ".join(statelist_asn1) \
+            or f'{process_asn1.lower()}-has-no-state'
+    # Create an ASN.1 definition of the list of states, instead of
+    # a native Ada type - this allows external tools to access
+    # information about the model without having to parse SDL
+    asn1_states_def = f"\n{process_asn1}-States ::= ENUMERATED" \
+                      f" {{{states_asn1}}}"
+
+    asn1_template.append(asn1_states_def)
+
+    # Template to generate the full SDL context as a single ASN.1 type
+    # This is very useful then to manipulate it from C, Ada or Python as
+    # it is then directly accessible and uPER-encodable
+    context_elems = []
+    if process.full_statelist:
+        # Some systems may have no states - only a start transition
+        context_elems.append(f'   state {process_asn1}-States')
+
+    context_elems.append ('init-done BOOLEAN')
+    # State aggregation: add list of substates
+    for substates in process.aggregates.values():
+        for each in substates:
+            context_elems.append(
+              f'{each.statename.lower()}{SEPARATOR}state {process_asn1}-States')
+
+    for var_name, (var_type, def_value) in process.variables.items():
+        if var_name in process.aliases.keys():
+            continue
+        try:
+            sortname = var_type.ReferencedTypeName
+        except AttributeError:
+            # if the DCL variable references a non-existing type, it shall
+            # not prevent the model saving.
+            continue
+        # If the type is a ends with _selection, i.e. it is an OG-created
+        # CHOICE selector, then prefix the type with the process name
+        if sortname.endswith('-selection'):
+            sortname = f'{process.name}-{sortname}'.title()
+        context_elems.append(f'{var_name.lower()} {sortname}')
+
+    asn1_context = (f'\n{process_asn1}-Context ::='' SEQUENCE {\n'
+                    + ",\n   ".join(line.replace("_", "-").replace("'", '"') for line in context_elems)
+                    + "\n}\n")
+
+    asn1_template.append(asn1_context)
+
+    # Add user-defined NEWTYPEs and CHOICE selector types
+    choice_selections = []
+    for sortname, sortdef in process.user_defined_types.items():
+        if sortdef.type.kind == "SequenceOfType":
+            rangeMin = sortdef.type.Min
+            rangeMax = sortdef.type.Max
+            refType  = sortdef.type.type.ReferencedTypeName
+            for refTypeCase in types_with_proper_case:
+                if refTypeCase.lower().replace('-', '_') == \
+                        refType.lower().replace('-', '_'):
+                    break
+            asn1_template.append(
+                    f'{sortname.replace("_", "-").capitalize()} ::= SEQUENCE '
+                    f'(SIZE ({rangeMin} .. {rangeMax})) OF '
+                    f'{refTypeCase.replace("_", "-")}')
+        elif sortdef.type.kind == "EnumeratedType":
+            # At the moment, the only user-defined ENUMERATED types that are
+            # supported are the ones generated for the CHOICE selectors
+            # We can therefore safely rename them systematically here,
+            # by using the process name as prefix, to avoid any risk of
+            # duplicate type definition in case it exists in another SDL
+            # process of the system.
+            prefixed_name = f'{process.name}_{sortdef.ChoiceTypeName}_selection '
+            keys = []
+            for idx, key in enumerate(sortdef.type.EnumValues.keys()):
+                # give an index to the enumerations to align with -selection
+                # types used in choice index
+                keys.append(f'{key}-present({idx+1})')
+            asn1_template.append(
+                    f'{prefixed_name.replace("_", "-").title()} ::= ENUMERATED {{' + ", ".join(keys) + '}')
+    asn1_template.append('END\n')
+
+    # Write the ASN.1 file
+    with open(process.name.lower() + '_datamodel.asn', 'w') as asn1_file:
+        asn1_file.write('\n'.join(asn1_template))
+
+
+
+def code_generation_preprocessing(process, separator=DEFAULT_SEPARATOR):
+    ''' Do all sorts of preprocessing before invoking a code generator
+        and update the AST of the process
+    '''
+    # In case model has nested states, flatten everything
+    flatten(process, sep=separator)
+
+    # Pre-processing of aliases: we must rename all references to the aliases
+    # when they point to a structure that contain a CHOICE field (we cannot
+    # use a rename clause in that case, since the field depends on a
+    # discriminant.
+    no_renames = []
+    for (alias, (sort, alias_expr)) in process.aliases.items():
+        def rec_detect_choice(expr: ogAST.Expression) -> bool:
+            if not isinstance(expr, ogAST.PrimSelector):
+                return False
+            receiver = expr.value[0]
+            bty = find_basic_type(TYPES=process.dataview,
+                                  a_type=receiver.exprType)
+            if bty.kind == 'ChoiceType':
+                return True
+            return rec_detect_choice(receiver)
+        is_choice = rec_detect_choice(alias_expr)
+        if is_choice:
+            LOG.debug(f"alias: {alias_expr.inputString} will replace {alias}")
+            rename_everything(process.content,
+                              alias,
+                              alias_expr.inputString)
+            no_renames.append(alias)
+
+    # Process State aggregations (Parallel states)
+
+    # Find recursively in the AST all state aggregations
+    # Format: {'aggregation_name' : [list of ogAST.CompositeState]
+    process.aggregates = state_aggregations(process)
+
+    # Make an mapping {input: {state: transition...}} in order to easily
+    # generate the lookup tables for the state machine runtime
+    process.input_mapping = map_input_state(process)
+
+    # Extract the list of parallel states names inside the composite states
+    # of state aggregations
+    process.parallel_states = parallel_states(process.aggregates)
+
+    # Establish the list of states (excluding START states)
+    update_full_statelist(process, separator)
+
+    return no_renames
