@@ -7,7 +7,7 @@
     This module generates textual SDL code (PR format)
     by parsing the graphical symbols.
 
-    Copyright (c) 2012-2016 European Space Agency
+    Copyright (c) 2012-2022 European Space Agency
 
     Designed and implemented by Maxime Perrotin
 
@@ -26,19 +26,23 @@ LOG = logging.getLogger(__name__)
 
 __all__ = ['parse_scene', 'generate']
 
+g_useSymbolId = False
 
 class Indent(deque):
-    ''' Extension of the deque class to support automatic indenting '''
+    ''' Extension of the deque class to support automatic indentation '''
     indent = 0
 
     def append(self, string):
         ''' Redefinition of the append to insert the indent pattern '''
-        super(Indent, self).append('    ' * Indent.indent + string)
+        super().append('    ' * Indent.indent + string)
 
 
-def parse_scene(scene, full_model=False):
+def parse_scene(scene, full_model=False, use_symbol_id=False):
     ''' Return the PR string for a complete scene
         Optionally, also generate the SYSTEM structure, with channels, etc. '''
+    global g_useSymbolId
+    # Decide to use the symbol ID rather than the CIF coordinates
+    g_useSymbolId = use_symbol_id
     pr_data = Indent()
     if full_model:
         # Generate a complete SDL system - to have everything in a single file
@@ -46,7 +50,7 @@ def parse_scene(scene, full_model=False):
         # (2) get signal directions from the connection of the process to env
         # (3) generate all the text
         processes = list(scene.processes)
-        system_name = str(processes[0]) if processes else u'OpenGEODE'
+        system_name = str(processes[0]) if processes else 'OpenGEODE'
         pr_data.append('system {};'.format(system_name))
         Indent.indent += 1
         channels, routes = Indent(), Indent()
@@ -55,7 +59,7 @@ def parse_scene(scene, full_model=False):
            pr = generate(each)
            pr_data.extend(pr)
         for pro in processes:
-            if isinstance(pro, sdlSymbols.ProcessType):
+            if isinstance(pro, sdlSymbols.ProcessType) or not pro.connection:
                 # process type does not have connections
                 continue
             to_env = pro.connection.out_sig
@@ -131,16 +135,25 @@ def parse_scene(scene, full_model=False):
 
 def cif_coord(name, symbol):
     ''' PR string for the CIF coordinates/size of a symbol '''
-    return u'/* CIF {symb} ({x}, {y}), ({w}, {h}) */'.format(
+    cif1 = '/* CIF {symb} ({x}, {y}), ({w}, {h}) */'.format(
             symb=name,
             x=int(symbol.scenePos().x()), y=int(symbol.scenePos().y()),
             w=int(symbol.boundingRect().width()),
             h=int(symbol.boundingRect().height()))
+    cif2 = '' if not g_useSymbolId else f'\n{cif_symbolid(symbol)}'
+    return cif1+cif2
+
+def cif_symbolid(symbol):
+    ''' CIF string returning the Python's identifier of a symbol, used only
+    when parsing an already rendered diagram. This allows to keep the link
+    between the re-parsed AST and the existing symbols, that is useful when
+    reporting errors '''
+    return f'/* CIF _id {id(symbol)} */'
 
 
 def hyperlink(symbol):
     ''' PR string for the optional hyperlink associated to a symbol '''
-    return u"/* CIF Keep Specific Geode HyperLink '{}' */".format(
+    return "/* CIF Keep Specific Geode HyperLink '{}' */".format(
                                                          symbol.text.hyperlink)
 
 
@@ -150,8 +163,8 @@ def common(name, symbol):
     result.append(cif_coord(name, symbol))
     if symbol.text.hyperlink:
         result.append(hyperlink(symbol))
-    result.append(u'{} {}{}'.format(name, str(symbol.text), ';'
-                                if not symbol.comment else ''))
+    result.append('{} {}{}'.format(name, str(symbol.text), ';'
+                               if not symbol.comment else ''))
     if symbol.comment:
         result.extend(generate(symbol.comment))
     return result
@@ -186,7 +199,7 @@ def _comment(symbol, **kwargs):
     result.append(cif_coord('comment', symbol))
     if symbol.text.hyperlink:
         result.append(hyperlink(symbol))
-    result.append(u'comment \'{}\';'.format(str(symbol.text)))
+    result.append("comment '" + str(symbol.text).replace("'", "\\'") + "';")
     return result
 
 
@@ -283,7 +296,7 @@ def _procedurecall(symbol, **kwargs):
     result.append(cif_coord('PROCEDURECALL', symbol))
     if symbol.text.hyperlink:
         result.append(hyperlink(symbol))
-    result.append(u'call {}{}'.format(str(symbol.text), ';'
+    result.append('call {}{}'.format(str(symbol.text), ';'
                                       if not symbol.comment else ''))
     if symbol.comment:
         result.extend(generate(symbol.comment))
@@ -300,7 +313,7 @@ def _textsymbol(symbol, **kwargs):
     # Align nicely the text (parser will dedent it)
     for line in str(symbol.text).split('\n'):
         result.append(line)
-    result.append(u'/* CIF ENDTEXT */')
+    result.append('/* CIF ENDTEXT */')
     return result
 
 
@@ -312,13 +325,13 @@ def _label(symbol, recursive=True, **kwargs):
     if symbol.text.hyperlink:
         result.append(hyperlink(symbol))
     if symbol.common_name == 'floating_label':
-        result.append(u'connection {}:'.format(str(symbol)))
+        result.append(f'connection {str(symbol)}:')
         if recursive:
             result.extend(recursive_aligned(symbol))
-        result.append(u'/* CIF End Label */')
-        result.append(u'endconnection;')
+        result.append('/* CIF End Label */')
+        result.append('endconnection;')
     else:
-        result.append(u'{}:'.format(str(symbol)))
+        result.append(f'{str(symbol)}:')
     return result
 
 
@@ -343,7 +356,7 @@ def _state(symbol, recursive=True, nextstate=True, composite=False, cpy=False,
                                               sdlSymbols.ContinuousSignal))):
                 result.extend(generate(each))
             Indent.indent -= 1
-        result.append(u'endstate;')
+        result.append('endstate;')
     else:
         # Generate code for a nested state
         result = Indent()
@@ -356,17 +369,17 @@ def _state(symbol, recursive=True, nextstate=True, composite=False, cpy=False,
             if str(each):
                 entry_points.append(str(each))
         for each in symbol.nested_scene.returns:
-            if str(each) != u'no_name':
+            if str(each) != 'no_name':
                 exit_points.append(str(each))
         if entry_points:
-            result.append(u'in ({});'.format(','.join(entry_points)))
+            result.append('in ({});'.format(','.join(entry_points)))
         if exit_points:
-            result.append(u'out ({});'.format(','.join(exit_points)))
+            result.append('out ({});'.format(','.join(exit_points)))
         Indent.indent += 1
-        result.extend(parse_scene(symbol.nested_scene))
+        result.extend(parse_scene(symbol.nested_scene, use_symbol_id=g_useSymbolId))
         Indent.indent -= 1
         Indent.indent -= 1
-        result.append(u'endsubstructure;')
+        result.append('endsubstructure;')
     return result
 
 
@@ -386,7 +399,7 @@ def _process(symbol, recursive=True, **kwargs):
 
     if recursive and symbol.nested_scene:
         Indent.indent += 1
-        result.extend(parse_scene(symbol.nested_scene))
+        result.extend(parse_scene(symbol.nested_scene, use_symbol_id=g_useSymbolId))
         Indent.indent -= 1
     if ":" not in str(symbol):
         result.append(u'endprocess {}{};'
@@ -402,7 +415,7 @@ def _procedure(symbol, recursive=True, **kwargs):
     result = common('procedure', symbol)
     if recursive and symbol.nested_scene:
         Indent.indent += 1
-        result.extend(parse_scene(symbol.nested_scene))
+        result.extend(parse_scene(symbol.nested_scene, use_symbol_id=g_useSymbolId))
         Indent.indent -= 1
     result.append('endprocedure;')
     return result
