@@ -3,7 +3,7 @@
 
 """
 
-    OpenGEODE - A tiny SDL Editor for TASTE
+    OpenGEODE - SDL Editor for TASTE
 
     AST that can be used to write SDL backends (code generators, etc.)
     In all classes the 'inputString' field corresponds to the exact
@@ -27,7 +27,7 @@
 
     See AdaGenerator.py for an example of use.
 
-    Copyright (c) 2012-2020 European Space Agency
+    Copyright (c) 2012-2022 European Space Agency
 
     Designed and implemented by Maxime Perrotin
 
@@ -39,6 +39,7 @@ import operator
 from collections import defaultdict
 LOG = logging.getLogger(__name__)
 
+from typing import List
 
 class Expression:
     ''' AST Entry for expressions - Always use subtype '''
@@ -61,9 +62,7 @@ class Expression:
 
     def trace(self):
         ''' Debug output for an expression '''
-        return u'{exp} ({l},{c})'.format(exp=self.inputString,
-                                        l=self.line,
-                                        c=self.charPositionInLine)
+        return f'{self.inputString} ({self.line},{self.charPositionInLine})'
 
 
 class ExprPlus(Expression):
@@ -254,18 +253,8 @@ class PrimStateReference(Primary):
     is_raw = False
 
 
-class PrimBitStringLiteral(Primary):
-    ''' Not supported yet '''
-    pass
-
-
-class PrimOctetStringLiteral(Primary):
-    ''' Not supported yet '''
-    pass
-
-
 class PrimConditional(Primary):
-    ''' value is a dictionnary:
+    ''' value is a dictionary:
         { 'if': Expression, 'then': Expression,
         'else': Expression, 'tmpVar': integer}
         tmpVar can be used if the backend needs a temporary variable
@@ -289,6 +278,19 @@ class PrimStringLiteral(Primary):
         self._exprType = val
 
 
+class PrimOctetStringLiteral(PrimStringLiteral):
+    ''' String of form 'ABCD'H '''
+    numeric_value : int = -1
+    # In case the string contains ASCII characters:
+    printable_string : str = '';
+    # set of hex numbers (e.g. 'FF'H means hexstring = b'\xff')
+    hexstring : bytes = b'';
+
+
+class PrimBitStringLiteral(PrimOctetStringLiteral):
+    ''' String of form '0001101'B '''
+    pass
+
 class PrimMantissaBaseExp(Primary):
     ''' Value is a dict: {'mantissa': int, 'base': int, 'exponent': int} '''
     pass
@@ -340,11 +342,15 @@ class Decision:
         self.hyperlink = None
         # hint for backends needing a temporary variable to hold the question
         self.tmpVar = -1
+        # Errors associated to this element
+        self.errors, self.warnings = [], []
+        # the path allows to retrieve the location of the symbol
+        # in the hierarchy (PROCESS foo, PROCEDURE bar..). for Error handling.
+        self.path = []
 
     def trace(self):
         ''' Debug output for a decision '''
-        return u'DECISION {exp} ({l},{c})'.format(exp=self.inputString,
-                l=self.line, c=self.charPositionInLine)
+        return f'DECISION {self.inputString} ({self.line},{self.charPositionInLine})'
 
 
 class Answer:
@@ -357,23 +363,29 @@ class Answer:
         self.pos_x, self.pos_y = None, None
         self.width = 70
         self.height = 23
-        # one of 'closed_range' 'constant' 'open_range' 'else' 'informal_text'
-        self.kind = None
-        # informalText is a string, when kind == 'informal_text'
-        self.informalText = None
-        # closedRange is a set of two numbers
-        self.closedRange = []
-        # constant is an Expression
-        #    (contains 'open_range' and 'constant' kinds corresponding value)
-        self.constant = None
-        # one of ExprEq, ExprNeq, ExprGt, ExprGe, ExprLt, ExprLe (types)
-        self.openRangeOp = None
+        # a single answer can contain several comma-separated statements
+        # to trigger a single transition from different options
+        # contents: list of dict :
+        # [{'kind': kind, 'content': content}] with kind =
+        #      'closed_range'|'constant'|'open_range'|'else'|'informal_text'
+        # and content is either strings (informal text)
+        #           or set of two numbers (closed range)
+        #           or tuple (EprEq, constant) for constant
+        #           or tuple (op, constant) for open_range
+        #           or None ('else' branch)
+        # with op being either ExprEq, ExprNeq, ExprGt, ExprGe, ExprLt, or ExprLe (types)
+        self.answers = []
         # transition is of type Transition
         self.transition : Transition = None
         # optional comment symbol
         self.comment = None
         # optional hyperlink
         self.hyperlink = None
+        # Errors associated to this element
+        self.errors, self.warnings = [], []
+        # the path allows to retrieve the location of the symbol
+        # in the hierarchy (PROCESS foo, PROCEDURE bar..). for Error handling.
+        self.path = []
 
     def trace(self):
         ''' Debug output for an answer '''
@@ -396,6 +408,11 @@ class Task:
         # optional hyperlink
         self.hyperlink = None
         self.elems = []
+        # Errors associated to this element
+        self.errors, self.warnings = [], []
+        # the path allows to retrieve the location of the symbol
+        # in the hierarchy (PROCESS foo, PROCEDURE bar..). for Error handling.
+        self.path = []
 
     def trace(self):
         ''' Debug output for a task '''
@@ -438,6 +455,11 @@ class Output:
         self.comment = None
         # optional hyperlink
         self.hyperlink = None
+        # Errors associated to this element
+        self.errors, self.warnings = [], []
+        # the path allows to retrieve the location of the symbol
+        # in the hierarchy (PROCESS foo, PROCEDURE bar..). for Error handling.
+        self.path = []
 
     def trace(self):
         ''' Debug output for an Output symbol '''
@@ -466,6 +488,9 @@ class Terminator:
         self.height = 35
         self.line = None
         self.charPositionInLine = None
+        # context is an ogAST entry pointing to the container of the terminator
+        # used for return statement to know the expected type
+        self.context = None
         # one of 'next_state' 'join' 'stop', 'return'
         self.kind = None
         # optional comment symbol
@@ -508,6 +533,11 @@ class Terminator:
         # states ; and there are the composite states, that set a different
         # id corresponding to the start transition of the state.
         self.candidate_id = defaultdict(list)
+        # Errors associated to this element
+        self.errors, self.warnings = [], []
+        # the path allows to retrieve the location of the symbol
+        # in the hierarchy (PROCESS foo, PROCEDURE bar..). for Error handling.
+        self.path = []
 
     def trace(self):
         ''' Debug output for terminators '''
@@ -537,6 +567,11 @@ class Label:
         self.terminators = []
         # Transition is used for floating labels (see AST entry)
         self.transition = None
+        # Errors associated to this element
+        self.errors, self.warnings = [], []
+        # the path allows to retrieve the location of the symbol
+        # in the hierarchy (PROCESS foo, PROCEDURE bar..). for Error handling.
+        self.path = []
 
     def trace(self):
         ''' Debug output for a label '''
@@ -548,7 +583,7 @@ class Floating_label(Label):
     ''' AST Entry for a floating label '''
     def __init__(self, label=None):
         ''' Initialize the floating label attributes '''
-        super(Floating_label, self).__init__()
+        super().__init__()
         if label:
             self.inputString = label.inputString
             self.pos_x = label.pos_x
@@ -619,10 +654,18 @@ class Input:
         self.hyperlink = None
         # list of terminators following the input symbol
         self.terminators = []
+        # If this input is in fact a continuous signal, code generators will ignore it
+        self.replaced_with_continuous_signal : bool = False
+        # Errors associated to this element
+        self.errors, self.warnings = [], []
+        # the path allows to retrieve the location of the symbol
+        # in the hierarchy (PROCESS foo, PROCEDURE bar..). for Error handling.
+        self.path = []
+
 
     def trace(self):
         ''' Debug output for an INPUT symbol '''
-        return u'INPUT {exp} ({l},{c})'.format(exp=self.inputString,
+        return 'INPUT {exp} ({l},{c})'.format(exp=self.inputString,
                 l=self.line, c=self.charPositionInLine)
 
 
@@ -630,7 +673,7 @@ class Connect(Input):
     ''' AST Entry for the CONNECT part (transition below a nested state) '''
     def __init__(self):
         ''' Only one difference with INPUT: the connect_list attribute '''
-        super(Connect, self).__init__()
+        super().__init__()
         # List of strings
         self.connect_list = []
         self.width = 1
@@ -644,11 +687,22 @@ class ContinuousSignal(Input):
     ''' AST Entry for the Continuous Signal '''
     def __init__(self):
         ''' Difference with Input: trigger is an expression '''
-        super(ContinuousSignal, self).__init__()
+        super().__init__()
         # Decision triggering the transition
         self.trigger = None
         # Priority (integer)
         self.priority = 0
+        # Set if we are in an observer to render the symbol differently
+        self.observer : bool = False
+        # instance of class Input when this CS is an alias of an input
+        self.observer_input = None
+        # artificial is set to True if this is an alias (for Renderer)
+        self.artificial = False
+        # Errors associated to this element
+        self.errors, self.warnings = [], []
+        # the path allows to retrieve the location of the symbol
+        # in the hierarchy (PROCESS foo, PROCEDURE bar..). for Error handling.
+        self.path = []
 
     def trace(self):
         ''' Debug output for a Continuous signal '''
@@ -672,10 +726,15 @@ class Start:
         self.hyperlink = None
         # list of terminators following the start symbol
         self.terminators = []
+        # Errors associated to this element
+        self.errors, self.warnings = [], []
+        # the path allows to retrieve the location of the symbol
+        # in the hierarchy (PROCESS foo, PROCEDURE bar..). for Error handling.
+        self.path = []
 
     def trace(self):
         ''' Debug output for a START symbol '''
-        return u'START {}'.format(self.inputString)
+        return f'START {self.inputString}'
 
 
 class Procedure_start(Start):
@@ -702,6 +761,11 @@ class Comment:
         self.charPositionInLine = None
         # optional hyperlink
         self.hyperlink = None
+        # Errors associated to this element
+        self.errors, self.warnings = [], []
+        # the path allows to retrieve the location of the symbol
+        # in the hierarchy (PROCESS foo, PROCEDURE bar..). for Error handling.
+        self.path = []
 
     def trace(self):
         ''' Debug output for a COMMENT symbol '''
@@ -741,6 +805,11 @@ class State:
         self.via = None
         # a state can be an instance of a state stype
         self.instance_of = None
+        # Errors associated to this element
+        self.errors, self.warnings = [], []
+        # the path allows to retrieve the location of the symbol
+        # in the hierarchy (PROCESS foo, PROCEDURE bar..). for Error handling.
+        self.path = []
 
     def trace(self):
         ''' Debug output for a STATE symbol '''
@@ -762,6 +831,8 @@ class TextArea:
         self.height = 140
         # DCL variables in the text area {name: (sort, default_value), ...}
         self.variables = {}
+        # MONITOR variables used in observers (same structure as DCL)
+        self.monitors = {}
         # fpar and timers are also listed, useful when using autocompletion
         self.fpar = []
         self.timers = []
@@ -775,6 +846,15 @@ class TextArea:
         self.procedures = []
         # optional hyperlink
         self.hyperlink = None
+        # List of Observer states defined in this text area (error/success/ignore states)
+        # used for error reporting to get the right symbol coordinates
+        self.observer_states : List [str] = []
+        # Errors associated to this element
+        self.errors, self.warnings = [], []
+        # the path allows to retrieve the location of the symbol
+        # in the hierarchy (PROCESS foo, PROCEDURE bar..). for Error handling.
+        self.path = []
+
 
     def trace(self):
         ''' Debug output for a text area '''
@@ -808,12 +888,15 @@ class Procedure:
         self.height = 35
         # Optional hyperlink
         self.hyperlink = None
-        # Local variables dictionnary (see Process)
+        # Local variables dictionary (see Process)
         self.variables = {}
+        # MONITOR variables used in observers (same structure as DCL)
+        self.monitors = {}
         self.timers = []
         # Inherited variables and timers from all levels above
         self.global_variables = {}
         self.global_timers = []
+        self.global_monitors = {}
         # Formal parameters - list of dict:
         # [{'name': str, 'direction': 'in'/'out', 'type': str}]
         self.fpar = []
@@ -840,8 +923,8 @@ class Procedure:
         self.transitions = []
         # Determine if a procedure is externally defined
         self.external = False
-        # Determine if a procedure is a remote procedure
-        self.exported = False
+        # Determine if a procedure only has a textual definition
+        self.textual_procedure = False
         # Optional comment
         self.comment = None
         # Set of symbols contained in the procedure (type Automaton)
@@ -851,6 +934,15 @@ class Procedure:
         self.output_signals = []
         # The "DECISION ANY" construct requires random number generators
         self.random_generator = set()
+        # Procedure declared as EXPORTED
+        self.exported : bool = False
+        # procedure declared as REFERENCED
+        self.referenced : bool = False
+        # Errors associated to this element
+        self.errors, self.warnings = [], []
+        # the path allows to retrieve the location of the symbol
+        # in the hierarchy (PROCESS foo, PROCEDURE bar..). for Error handling.
+        self.path = []
 
 
 class Process:
@@ -869,11 +961,22 @@ class Process:
         self.parent = None
         # A process can be referenced (externally defined)
         self.referenced = False
-        # variables: dictionnary: {variable1Name: (asn1SccType, default value)}
+        # variables: dictionary: {variable1Name: (asn1SccType, default value)}
         self.variables = {}
+        # MONITOR variables used in observers (same structure as DCL)
+        self.monitors = {}
         # global variables and timers can be used to inherit from a level above
         self.global_variables = {}
+        self.global_monitors = {}
         self.global_timers = []
+        # aliases allow to access deep structures fields with a shorter syntax
+        # the _ast version is a temporary storage of the antlr parse tree. The actual
+        # processed entry is created from it after all actual variables from all
+        # text areas have been parsed, to be able to check the type of the alias
+        # format of the _ast: list of tuple (variable name, asn1 sort, ANTLR ast, ogAST.Text_Area)
+        self._aliases_ast = []
+        # format: {'alias_name': (asn1 sort, ogAST.Expression)
+        self.aliases = {}
         # Set default coordinates and width/height
         self.pos_x = 250
         self.pos_y = 150
@@ -910,12 +1013,15 @@ class Process:
         self.procedures = []
 
         # The Mapping structure should be used for code generation backends
-        # dictionnary: {'stateName': [class Input1, Input2,...], ...}
+        # dictionary: {'stateName': [class Input1, Input2,...], ...}
         # then Input contains the inputs list and corresponding transition
         self.mapping = {}
 
         # Similar mapping for continuous signals
         self.cs_mapping = defaultdict(list)
+        
+        # Similar mapping for CONNECT triggers
+        self.connect_mapping = defaultdict(list)
 
         # list of type Transition - use 'mapping' to map index to inputs/states
         self.transitions = []
@@ -940,6 +1046,44 @@ class Process:
         # The "DECISION ANY" construct requires random number generators
         self.random_generator = set()
 
+        # list of Error, Success, and Ignore states (observers/model checking)
+        # (strings in lowercase)
+        self.errorstates = []
+        self.ignorestates = []
+        self.successstates = []
+        # Errors associated to this element
+        self.errors, self.warnings = [], []
+        # the path allows to retrieve the location of the symbol
+        # in the hierarchy (PROCESS foo, PROCEDURE bar..). for Error handling.
+        self.path = []
+
+        # look-up table to find the module name containing a type
+        # (filled by the "generate_asn1_datamodel" function)
+        self.mapping_sort_module = dict()
+
+        # Full list of state excluding START states
+        # (filled by Helper.update_full_statelist)
+        self.full_statelist = set()
+
+        # name: usable process name, it can be different from processName
+        # if this is a process type and the code generation is both for the
+        # type and its instance.
+        self.name : str = ''
+
+        # Format: {'aggregation_name' : [list of ogAST.CompositeState]
+        # (filled by Helper)
+        self.aggregates = dict()
+
+        # Input mapping
+        # {input: {state: transition...}} in order to easily
+        # generate the lookup tables for the state machine runtime
+        # (Filled by Helper)
+        self.input_mapping = dict()
+
+        # List of parallel states names inside the composite states
+        # of state aggregations (Filled by helper)
+        self.parallel_states = []
+
 
 class CompositeState(Process):
     '''
@@ -949,7 +1093,7 @@ class CompositeState(Process):
         - entry and exit procedures
     '''
     def __init__(self):
-        super(CompositeState, self).__init__()
+        super().__init__()
         self.statename = ''
         self.state_entrypoints = set()
         self.state_exitpoints = set()
@@ -962,13 +1106,14 @@ class CompositeState(Process):
 
     def trace(self):
         ''' Debug output for composite state '''
-        return u'COMPOSITE STATE {exp} ({l},{c})'.format(exp=self.statename,
+        return 'COMPOSITE STATE {exp} ({l},{c})'.format(exp=self.statename,
                 l=self.line, c=self.charPositionInLine)
 
 
 class StateAggregation(CompositeState):
     '''
         State Aggregation (Parallel states) are supported since SDL2000
+
         These states can only contain (in the self.content field):
             text areas
             procedure definitions
@@ -976,7 +1121,7 @@ class StateAggregation(CompositeState):
         But no state machine definition
     '''
     def __init__(self):
-        super(StateAggregation, self).__init__()
+        super().__init__()
         # List of partition connections:
         # [{'outer': {'state_part_id': str, 'point': str},
         #   'inner': {'state_part_id': str, 'point': str}}]
@@ -1032,6 +1177,7 @@ class System:
         self.filename = None
         # Reference to top-level AST
         self.ast = None
+        # self.parent = None
         # list of SIGNAL declarations: [{'name': str, 'type': asn1type}]
         # (Supporting only one parameter)
         self.signals = []
@@ -1042,6 +1188,7 @@ class System:
         self.channels = []
         self.blocks = []
         self.text_areas = []
+        self.path = []
 
 
 class AST:
@@ -1051,6 +1198,8 @@ class AST:
             AST entries are systems, processes, and USE clauses
             (cf. ANTLR grammar, production "pr_file")
         '''
+        # Top-level node: parent remains None
+        self.parent = None
         # Set of input files the AST was created from
         self.pr_files = set()
         # USE clauses: list of strings (eg. "DataView")
@@ -1063,7 +1212,7 @@ class AST:
         # DV is the Asn1scc imported module
         self.DV = None
         # ASN.1-defined constants (constants in Ada but variables in C)
-        # dictionnary: {ConstantName: type } - copied from dataview.py
+        # dictionary: {ConstantName: type } - copied from dataview.py
         self.asn1_constants = {}
 
         # List of System
