@@ -10,7 +10,7 @@
     All symbols inherit the generic Vertical- and Horizontal-
     Symbol classes defined in the "genericSymbols.py" module.
 
-    Copyright (c) 2012-2020 European Space Agency
+    Copyright (c) 2012-2021 European Space Agency
 
     Designed and implemented by Maxime Perrotin
 
@@ -26,9 +26,9 @@ import traceback
 import logging
 from itertools import chain
 
-from PySide2.QtCore import *
-from PySide2.QtGui import *
-from PySide2.QtWidgets import *
+from PySide6.QtCore import *
+from PySide6.QtGui import *
+from PySide6.QtWidgets import *
 
 from .genericSymbols import HorizontalSymbol, VerticalSymbol, Comment
 from .Connectors import Connection, JoinConnection, Signalroute
@@ -51,6 +51,8 @@ SDL_BLACKBOLD = ['\\b{word}\\b'.format(word=word) for word in (
                 'SET_TIMER', 'RESET_TIMER', 'VIA', 'ENTRY', 'EXIT', 'PRIORITY',
                 'SYNTYPE', 'ENDSYNTYPE', 'CONSTANTS', 'ENDPROCEDURE', 'FOR',
                 'COMMENT', 'SIGNAL', 'SIGNALLIST', 'USE', 'RETURNS', 'ANY',
+                'EXPORTED', 'REFERENCED', 'MONITOR', 'RENAMES',
+                'SUCCESSSTATES', 'ERRORSTATES', 'IGNORESTATES',
                 'NEWTYPE', 'ENDNEWTYPE', 'ARRAY', 'STRUCT', 'SYNONYM')]
 
 SDL_REDBOLD = ['\\b{word}\\b'.format(word=word) for word in (
@@ -66,7 +68,8 @@ def variables_autocompletion(symbol, type_filter=None):
     res = set()
     if not symbol.text:
         return res
-    parts = symbol.text.context.split('!')
+    sep = "!" if "!" in symbol.text.context else "."
+    parts = symbol.text.context.split(sep)
     if len(parts) == 0:
         return res
     elif len(parts) == 1:
@@ -75,7 +78,7 @@ def variables_autocompletion(symbol, type_filter=None):
         except AttributeError:
             # not in the context of a procedure
             fpar = {}
-        # Return the list of variables, possibly filterd by type
+        # Return the list of variables, possibly filtered by type
         if not type_filter:
             res = set( list(CONTEXT.variables.keys())
                       + list(CONTEXT.global_variables.keys())
@@ -146,6 +149,9 @@ class Input(HorizontalSymbol):
     blackbold = SDL_BLACKBOLD
     redbold = SDL_REDBOLD
 
+    # Minimum size for symbol
+    min_size = (70, 35)
+
     def __init__(self, parent=None, ast=None):
         ''' Create the INPUT symbol '''
         ast = ast or ogAST.Input()
@@ -155,7 +161,7 @@ class Input(HorizontalSymbol):
         if not ast.pos_y and parent:
             # Make sure the item is placed below its parent
             ast.pos_y = parent.y() + parent.boundingRect().height() + 10
-        super(Input, self).__init__(parent,
+        super().__init__(parent,
                                     text=ast.inputString,
                                     x=ast.pos_x or 0,
                                     y=ast.pos_y or 0,
@@ -177,7 +183,7 @@ class Input(HorizontalSymbol):
                                                          ContinuousSignal))
                        else parent.parentItem())
         self.branch_entrypoint = item_parent.branch_entrypoint
-        super(Input, self).insert_symbol(item_parent, x, y)
+        super().insert_symbol(item_parent, x, y)
 
     def boundingRect(self):
         return QRectF(0, 0, self.width, self.height)
@@ -193,7 +199,7 @@ class Input(HorizontalSymbol):
             path.lineTo(0, height)
             path.lineTo(0, 0)
             self.setPath(path)
-            super(Input, self).set_shape(width, height)
+            super().set_shape(width, height)
 
     @property
     def completion_list(self):
@@ -213,7 +219,6 @@ class Input(HorizontalSymbol):
 class Connect(Input):
     ''' Connect point below a nested state '''
     common_name = 'connect_part'
-    auto_expand = True
     resizeable = False
     # Symbol must not use antialiasing, otherwise the middle line is too thick
     _antialiasing = False
@@ -258,11 +263,14 @@ class Output(VerticalSymbol):
     blackbold = SDL_BLACKBOLD
     redbold = SDL_REDBOLD
 
+    # Minimum size for symbol
+    min_size = (70, 35)
+
     def __init__(self, parent=None, ast=None):
         ast = ast or ogAST.Output()
         self.ast = ast
         self.width, self.height = 0, 0
-        super(Output, self).__init__(parent=parent,
+        super().__init__(parent=parent,
                 text=ast.inputString, x=ast.pos_x or 0, y=ast.pos_y or 0,
                 hyperlink=ast.hyperlink)
         self.set_shape(ast.width, ast.height)
@@ -283,7 +291,7 @@ class Output(VerticalSymbol):
             path.lineTo(0, height)
             path.lineTo(0, 0)
             self.setPath(path)
-            super(Output, self).set_shape(width, height)
+            super().set_shape(width, height)
 
     @property
     def completion_list(self):
@@ -312,13 +320,16 @@ class Decision(VerticalSymbol):
                                    for word in ('AND', 'OR')]
     redbold = SDL_REDBOLD
 
+    # Minimum size for symbol
+    min_size = (70, 50)
+
     def __init__(self, parent=None, ast=None):
         ast = ast or ogAST.Decision()
         self.ast = ast
         self.width, self.height = 0, 0
         # Define the point where all branches of the decision can join again
         self.connectionPoint = QPoint(ast.width / 2, ast.height + 30)
-        super(Decision, self).__init__(parent, text=ast.inputString,
+        super().__init__(parent, text=ast.inputString,
                 x=ast.pos_x or 0, y=ast.pos_y or 0, hyperlink=ast.hyperlink)
         self.set_shape(ast.width, ast.height)
         self.setBrush(QColor(255, 255, 202))
@@ -359,22 +370,23 @@ class Decision(VerticalSymbol):
             path.lineTo(0, height / 2)
             path.lineTo(width / 2, 0)
             self.setPath(path)
-            super(Decision, self).set_shape(width, height)
+            super().set_shape(width, height)
 
     def resize_item(self, rect):
         ''' On resize event, make sure connection points are updated '''
         delta_y = self.boundingRect().height() - rect.height()
-        super(Decision, self).resize_item(rect)
+        super().resize_item(rect)
         self.connectionPoint.setX(self.boundingRect().center().x())
         self.connectionPoint.setY(self.connectionPoint.y() - delta_y)
         self.update_connections()
 
     def update_connections(self):
         ''' Redefined - update arrows shape below connection point '''
-        super(Decision, self).update_connections()
+        super().update_connections()
         for branch in self.branches():
             for cnx in branch.last_branch_item.connections():
                 cnx.reshape()
+        self.updateConnectionPointPosition()
 
     def updateConnectionPointPosition(self):
         ''' Compute the joining point of decision branches '''
@@ -434,7 +446,7 @@ class Decision(VerticalSymbol):
                 child.pos_y += delta
             except AttributeError:
                 pass
-        self.update_connections()
+        #self.update_connections()
 
 
 # pylint: disable=R0904
@@ -448,31 +460,34 @@ class DecisionAnswer(HorizontalSymbol):
     blackbold = SDL_BLACKBOLD
     redbold = SDL_REDBOLD
 
+    # Minimum size for symbol
+    min_size = (70, 23)
+
     def __init__(self, parent=None, ast=None):
         ast = ast or ogAST.Answer()
         self.ast = ast
         self.width, self.height = 0, 0 #ast.width, ast.height
         self.terminal_symbol = False
         # last_branch_item is used to compute branch length
-        # for the connection point positionning
+        # for the connection point positioning
         self.last_branch_item = self
-        super(DecisionAnswer, self).__init__(parent,
-                                             text=ast.inputString,
-                                             x=ast.pos_x or 0,
-                                             y=ast.pos_y or 0,
-                                             hyperlink=ast.hyperlink)
+        super().__init__(parent,
+                         text=ast.inputString,
+                         x=ast.pos_x or 0,
+                         y=ast.pos_y or 0,
+                         hyperlink=ast.hyperlink)
         self.set_shape(ast.width, ast.height)
         self.branch_entrypoint = self
         self.parser = ogParser
 
     def insert_symbol(self, parent, x, y):
-        ''' ANSWER-specific insersion behaviour: link to connection point '''
+        ''' ANSWER-specific insertion behaviour: link to connection point '''
         if not parent:
             return
         # Make sure that parent is not a sibling answer
         item_parent = (parent if not isinstance(parent, DecisionAnswer)
                        else parent.parentItem())
-        super(DecisionAnswer, self).insert_symbol(item_parent, x, y)
+        super().insert_symbol(item_parent, x, y)
         self.last_branch_item.connectionBelow = \
                 JoinConnection(self.last_branch_item, item_parent)
         self.text.try_resize()
@@ -493,7 +508,7 @@ class DecisionAnswer(HorizontalSymbol):
             path.arcTo(right, -55, 110)
             path.moveTo(width, height)
             self.setPath(path)
-            super(DecisionAnswer, self).set_shape(width, height)
+            super().set_shape(width, height)
 
     @property
     def completion_list(self):
@@ -504,13 +519,15 @@ class DecisionAnswer(HorizontalSymbol):
 # pylint: disable=R0904
 class Join(VerticalSymbol):
     ''' JOIN symbol (GOTO) '''
-    auto_expand = True
     arrow_head = 'simple'
     common_name = 'terminator_statement'
     # Define reserved keywords for the syntax highlighter
     blackbold = SDL_BLACKBOLD
     redbold = SDL_REDBOLD
 
+    # Minimum size for symbol
+    min_size = (70, 35)
+
     def __init__(self, parent=None, ast=None):
         self.ast = ast
         self.width, self.height = 0, 0
@@ -519,7 +536,7 @@ class Join(VerticalSymbol):
             ast.pos_y = 0
             ast.width = 35
             ast.height = 35
-        super(Join, self).__init__(parent,
+        super().__init__(parent,
                                    text=ast.inputString,
                                    x=ast.pos_x,
                                    y=ast.pos_y,
@@ -534,16 +551,16 @@ class Join(VerticalSymbol):
         size = min(rect.width(), rect.height())
         rect.setWidth(size)
         rect.setHeight(size)
-        super(Join, self).resize_item(rect)
+        super().resize_item(rect)
 
     def set_shape(self, width, height):
-        ''' Define the bouding rectangle of the JOIN symbol '''
+        ''' Define the bounding rectangle of the JOIN symbol '''
         if width != self.width or height != self.height:
             circ = min(width, height)
             path = QPainterPath()
             path.addEllipse(0, 0, circ, circ)
             self.setPath(path)
-            super(Join, self).set_shape(width, height)
+            super().set_shape(width, height)
 
     @property
     def completion_list(self):
@@ -579,7 +596,7 @@ class ProcedureStop(Join):
             ast.pos_y = 0
             ast.width = 35
             ast.height = 35
-        super(ProcedureStop, self).__init__(parent, ast)
+        super().__init__(parent, ast)
 
     def set_shape(self, width, height):
         ''' Define the symbol shape '''
@@ -635,11 +652,14 @@ class Label(VerticalSymbol):
     # Symbol must not use antialiasing, otherwise the middle line is too thick
     _antialiasing = False
 
+    # Minimum size for symbol
+    min_size = (70, 35)
+
     def __init__(self, parent=None, ast=None):
         ast = ast or ogAST.Label()
         self.ast = ast
         self.width, self.height = 0, 0
-        super(Label, self).__init__(parent,
+        super().__init__(parent,
                                     text=ast.inputString,
                                     x=ast.pos_x or 0,
                                     y=ast.pos_y or 0,
@@ -657,21 +677,22 @@ class Label(VerticalSymbol):
     def set_shape(self, width, height):
         ''' Define the shape of the LABEL symbol '''
         if width != self.width or height != self.height:
+            circ = 35 / 2
             path = QPainterPath()
-            path.addEllipse(0, height / 2, width / 4, height / 2)
-            path.moveTo(width / 4, height * 3 / 4)
-            path.lineTo(width / 2, height * 3 / 4)
+            path.addEllipse(width / 2 - 35, circ, circ, circ)
+            path.moveTo(width / 2 - circ, 35 * 3 / 4)
+            path.lineTo(width / 2, 35 * 3 / 4)
             # Add arrow head
-            path.moveTo(width / 2 - 5, height * 3 / 4 - 5)
-            path.lineTo(width / 2, height * 3 / 4)
-            path.lineTo(width / 2 - 5, height * 3 / 4 + 5)
+            path.moveTo(width / 2 - 5, 35 * 3 / 4 - 5)
+            path.lineTo(width / 2, 35 * 3 / 4)
+            path.lineTo(width / 2 - 5, 35 * 3 / 4 + 5)
             # Add vertical line in the middle of the symbol
             path.moveTo(width / 2, 0)
             path.lineTo(width / 2, height)
             # Make sure the bounding rect is withing specifications
             path.moveTo(width, height)
             self.setPath(path)
-            super(Label, self).set_shape(width, height)
+            super().set_shape(width, height)
 
     @property
     def completion_list(self):
@@ -706,12 +727,15 @@ class Task(VerticalSymbol):
     blackbold = SDL_BLACKBOLD
     redbold = SDL_REDBOLD
 
+    # Minimum size for symbol
+    min_size = (70, 35)
+
     def __init__(self, parent=None, ast=None):
         ''' Initializes the TASK symbol '''
         ast = ast or ogAST.Task()
         self.ast = ast
         self.width, self.height = 0, 0
-        super(Task, self).__init__(parent,
+        super().__init__(parent,
                                    text=ast.inputString,
                                    x=ast.pos_x or 0,
                                    y=ast.pos_y or 0,
@@ -732,7 +756,7 @@ class Task(VerticalSymbol):
             path.lineTo(0, height)
             path.lineTo(0, 0)
             self.setPath(path)
-            super(Task, self).set_shape(width, height)
+            super().set_shape(width, height)
 
     @property
     def completion_list(self):
@@ -772,11 +796,14 @@ class ProcedureCall(VerticalSymbol):
                  '\\bSET_TIMER\\b', '\\bRESET_TIMER\\b']
     redbold = SDL_REDBOLD
 
+    # Minimum size for symbol
+    min_size = (70, 35)
+
     def __init__(self, parent=None, ast=None):
         ast = ast or ogAST.Output(defName='')
         self.ast = ast
         self.width, self.height = 0, 0
-        super(ProcedureCall, self).__init__(parent,
+        super().__init__(parent,
                                             text=ast.inputString,
                                             x=ast.pos_x or 0,
                                             y=ast.pos_y or 0,
@@ -798,7 +825,7 @@ class ProcedureCall(VerticalSymbol):
             path.moveTo(width - 7, 0)
             path.lineTo(width - 7, height)
             self.setPath(path)
-            super(ProcedureCall, self).set_shape(width, height)
+            super().set_shape(width, height)
 
     @property
     def completion_list(self):
@@ -843,12 +870,15 @@ class TextSymbol(HorizontalSymbol):
     blackbold = SDL_BLACKBOLD
     redbold = SDL_REDBOLD
 
+    # Minimum size for symbol
+    min_size = (170, 140)
+
     def __init__(self, ast=None):
         ''' Create a Text Symbol '''
         ast = ast or ogAST.TextArea()
         self.ast = ast
         self.width, self.height = 0, 0
-        super(TextSymbol, self).__init__(parent=None,
+        super().__init__(parent=None,
                                          text=ast.inputString,
                                          x=ast.pos_x or 0,
                                          y=ast.pos_y or 0,
@@ -869,7 +899,7 @@ class TextSymbol(HorizontalSymbol):
         # a semi-colon, since that is always the case with declarations
         # and the text box cannot be followed by a COMMENT symbol
         return super().check_syntax(pr_text, check_last_semi=False);
-        
+
 
     def update_completion_list(self, pr_text):
         ''' When text was entered, update list of variables/FPAR/Timers '''
@@ -928,7 +958,7 @@ class TextSymbol(HorizontalSymbol):
             path.lineTo(width - 10, 0)
             path.lineTo(width, 10)
             self.setPath(path)
-            super(TextSymbol, self).set_shape(width, height)
+            super().set_shape(width, height)
 
     def resize_item(self, rect):
         ''' Text Symbol only resizes down or right '''
@@ -951,12 +981,15 @@ class State(VerticalSymbol):
     redbold = SDL_REDBOLD
     context_name = "state"
 
+    # Minimum size for symbol
+    min_size = (70, 35)
+
     def __init__(self, parent=None, ast=None):
         ast = ast or ogAST.State()
         self.ast = ast
         self.width, self.height = 0, 0
         ast.inputString = getattr(ast, 'via', None) or ast.inputString
-        super(State, self).__init__(parent=parent,
+        super().__init__(parent=parent,
                                     text=ast.inputString,
                                     x=ast.pos_x or 0,
                                     y=ast.pos_y or 0,
@@ -1007,7 +1040,7 @@ class State(VerticalSymbol):
 
     def update_completion_list(self, pr_text):
         ''' When text was entered, update state completion list '''
-        # Get AST for the symbol and update the context dictionnary
+        # Get AST for the symbol and update the context dictionary
         ast, _, _, _, _ = self.parser.parseSingleElement('state', pr_text)
         if ast:
             # None if there were syntax errors in the symbol
@@ -1046,7 +1079,7 @@ class State(VerticalSymbol):
             path = QPainterPath()
             path.addRoundedRect(0, 0, width, height, height / 4, height)
             self.setPath(path)
-            super(State, self).set_shape(width, height)
+            super().set_shape(width, height)
 
     def get_ast(self, pr_text):
         ''' Redefinition of the get_ast function for the state '''
@@ -1073,7 +1106,7 @@ class Process(HorizontalSymbol):
     blackbold = SDL_BLACKBOLD
     redbold = SDL_REDBOLD
     completion_list = set()
-    is_singleton = True
+    is_singleton = True #(False to allow multiple processes)
     arrow_head = 'angle'
     arrow_tail = 'angle'
     # Process can be connected to other processes by the user
@@ -1082,7 +1115,12 @@ class Process(HorizontalSymbol):
     _conn_targets = ['Process']
     context_name = "process"
 
-    def __init__(self, ast=None, subscene=None):
+    # Minimum size for symbol
+    min_size = (150, 75)
+
+    def __init__(self,
+                 ast=None,
+                 subscene=None):
         ast = ast or ogAST.Process()
         self.ast = ast
         self.width, self.height = 0, 0
@@ -1094,11 +1132,11 @@ class Process(HorizontalSymbol):
         # makes a call to resize_item, which calls set_shape using a size
         # defined by the label. set shape will then be called again using
         # the ast-defined size.
-        super(Process, self).__init__(parent=None,
-                                      text=label,
-                                      x=ast.pos_x,
-                                      y=ast.pos_y,
-                                      hyperlink=ast.hyperlink)
+        super().__init__(parent=None,
+                         text=label,
+                         x=ast.pos_x,
+                         y=ast.pos_y,
+                         hyperlink=ast.hyperlink)
         self.set_shape(ast.width, ast.height)
         self.setBrush(QBrush(QColor(255, 255, 202)))
         self.parser = ogParser
@@ -1126,11 +1164,11 @@ class Process(HorizontalSymbol):
         yield QRect(15, 5, rect.width() - 30, 10)
         yield QRect(5, 5, 10, rect.height() - 10)
         yield QRect(rect.width() - 15, 5, 10, rect.height() - 10)
-        yield QRect(15, rect.heigth() - 15, rect.width() - 30, 10)
+        yield QRect(15, rect.height() - 15, rect.width() - 30, 10)
 
     def insert_symbol(self, parent, x, y):
         ''' Redefinition - adds connection line to env '''
-        super(Process, self).insert_symbol(parent, x, y)
+        super().insert_symbol(parent, x, y)
         if not self.connection:
             self.connection = self.connect_to_parent()
 
@@ -1154,7 +1192,27 @@ class Process(HorizontalSymbol):
             path.lineTo(width - 7, 0)
             path.lineTo(7, 0)
             self.setPath(path)
-            super(Process, self).set_shape(width, height)
+            super().set_shape(width, height)
+
+    def cam_group(self):
+        ''' Redefine the graphical boundaries of the item to apply the CAM
+            If process has child connections (connections to other processes)
+            the CAM group should only include the process block itself,
+            not all the lines around it '''
+        return self.sceneBoundingRect()
+
+    def mouse_move(self, event):
+        ''' In addition to default behaviour: update channel connections '''
+        #super().mouse_move(event)
+        if self.mode == 'Move':
+            event_pos = event.pos()
+            new_y = self.pos_y + (event_pos.y() - event.lastPos().y())
+            new_x = self.pos_x + (event_pos.x() - event.lastPos().x())
+            self.position = QPointF(new_x, new_y)
+            # Signal the move to the connections
+            self.moved.emit(event.lastPos().x() - event.pos().x(),
+                            event.lastPos().y() - event.pos().y())
+
 
     def update_completion_list(self, pr_text):
         ''' When text was entered, update completion list at block level '''
@@ -1181,6 +1239,9 @@ class Procedure(Process):
     user_can_connect = False
     context_name = "procedure"
 
+    # Minimum size for symbol
+    min_size = (70, 35)
+
     def __init__(self, ast=None, subscene=None):
         ast = ast or ogAST.Procedure()
         self.ast = ast
@@ -1240,6 +1301,9 @@ class ProcessType(Procedure):
     completion_list = set()
     is_singleton = False
     user_can_connect = False
+    
+    # Minimum size for symbol
+    min_size = (150, 75)
 
     def __init__(self, ast=None, subscene=None):
         ast = ast or ogAST.Process()
@@ -1286,7 +1350,6 @@ class ProcessType(Procedure):
             self.setPath(path)
             super(Process, self).set_shape(width, height)
 
-
     def update_completion_list(self, pr_text):
         ''' After text is entered in process type, don't update any context '''
         # Todo perhaps later for autocompletion of process instances
@@ -1308,13 +1371,16 @@ class Start(HorizontalSymbol):
     redbold = SDL_REDBOLD
     has_text_area = False
 
+    # Minimum size for symbol
+    min_size = (70, 35)
+
     def __init__(self, ast=None):
         ''' Create the START symbol '''
         ast = ast or ogAST.Start()
         self.ast = ast
         self.terminal_symbol = False
         self.width, self.height = 0, 0
-        super(Start, self).__init__(parent=None,
+        super().__init__(parent=None,
                                     text=ast.inputString[slice(0, -6)],
                                     x=ast.pos_x or 0,
                                     y=ast.pos_y or 0,
@@ -1337,7 +1403,7 @@ class Start(HorizontalSymbol):
             path = QPainterPath()
             path.addRoundedRect(0, 0, width, height, height / 2, height / 2)
             self.setPath(path)
-            super(Start, self).set_shape(width, height)
+            super().set_shape(width, height)
 
 
 class ProcedureStart(Start):
@@ -1373,7 +1439,7 @@ class StateStart(Start):
     def update_completion_list(self, pr_text):
         ''' Update nested state entry points '''
         CONTEXT.state_entrypoints.add(str(self))
-        
+
 
 # pylint: disable=R0904
 class ContinuousSignal(HorizontalSymbol):
@@ -1384,8 +1450,19 @@ class ContinuousSignal(HorizontalSymbol):
     _terminal_followers = ['Join', 'State', 'ProcedureStop']
     common_name = 'continuous_signal'
     # Define reserved keywords for the syntax highlighter
-    blackbold = SDL_BLACKBOLD
-    redbold = SDL_REDBOLD
+    blackbold = SDL_BLACKBOLD.copy()
+    redbold = SDL_REDBOLD.copy()
+    redbold.remove("\\bINPUT\\b")
+    redbold.remove("\\bOUTPUT\\b")
+    redbold.remove("\\bSTATE\\b")
+    blackbold.append("\\bINPUT\\b")
+    blackbold.append("\\bOUTPUT\\b")
+    blackbold.append("\\bAND\\b")
+    blackbold.append("\\bFROM\\b")
+    blackbold.append("\\bTO\\b")
+
+    # Minimum size for symbol
+    min_size = (70, 35)
 
     def __init__(self, parent=None, ast=None):
         ''' Create the Provided symbol - use no background color '''
@@ -1396,7 +1473,7 @@ class ContinuousSignal(HorizontalSymbol):
         if not ast.pos_y and parent:
             # Make sure the item is placed below its parent
             ast.pos_y = parent.y() + parent.boundingRect().height() + 10
-        super(ContinuousSignal, self).__init__(parent, text=ast.inputString,
+        super().__init__(parent, text=ast.inputString,
                 x=ast.pos_x or 0, y=ast.pos_y or 0, hyperlink=ast.hyperlink)
         self.set_shape(ast.width, ast.height)
         self.terminal_symbol = False
@@ -1411,7 +1488,7 @@ class ContinuousSignal(HorizontalSymbol):
                                                          ContinuousSignal))
                        else parent.parentItem())
         self.branch_entrypoint = item_parent.branch_entrypoint
-        super(ContinuousSignal, self).insert_symbol(item_parent, x, y)
+        super().insert_symbol(item_parent, x, y)
 
     def set_shape(self, width, height):
         ''' Define the shape '''
@@ -1420,15 +1497,24 @@ class ContinuousSignal(HorizontalSymbol):
             path.moveTo(15, 0)
             path.lineTo(0, height / 2.0)
             path.lineTo(15, height)
-            path.moveTo(width - 15, 0)
+            is_observer = self.ast.observer
+            if isinstance(CONTEXT, ogAST.Process) and CONTEXT.monitors:
+                is_observer = True
+            if is_observer:
+                self.setBrush(QBrush(QColor(255, 248, 221)))
+                # If there are monitors, it is an observer
+                # -> change slightly the symbol shape
+                path.lineTo(width - 15, height)
+            else:
+                path.moveTo(width - 15, height)
             path.lineTo(width, height / 2.0)
-            path.lineTo(width - 15, height)
+            path.lineTo(width - 15, 0)
+            if is_observer:
+                path.lineTo(15, 0)
             self.setPath(path)
-            super(ContinuousSignal, self).set_shape(width, height)
+            super().set_shape(width, height)
 
     @property
     def completion_list(self):
         ''' Set auto-completion list '''
         return variables_autocompletion(self, None)
-
-
